import logging

from django.core.management.base import BaseCommand
from django.contrib.auth import load_backend, login
from django.contrib.auth.backends import RemoteUserBackend
from django.contrib.auth import get_user_model
from django_auth_ldap.config import LDAPSearch
import django.conf

def execute_ldap_search_without_hiding_errors(search, connection, filterargs=(), escape=True):
	'''If an LDAPError occurs during search.execute, it logs the error and returns
	an empty list of results. This behavor is indistinguishable from a query
	with no results.'''
	if not isinstance(search, LDAPSearch):
		raise NotImplementedError(f'{type(search)} is not supported by django_auth_ldap_remoteuser')
	# This is a copy of django_auth_ldap.config.LDAPSearch.execute without the
	# ldap.LDAPError eating try-except block
	if escape:
		filterargs = search._escape_filterargs(filterargs)
	filterstr = search.filterstr % filterargs
	results = connection.search_s(search.base_dn, search.scope, filterstr, search.attrlist)
	return search._process_results(results)

class Command(BaseCommand):
	can_import_settings = True
	help = 'Synchronize users from a LDAP server'

	def handle(self, *args, **options):
		ldap_backend = load_backend('django_auth_ldap.backend.LDAPBackend')
		conn = ldap_backend.ldap.initialize(ldap_backend.settings.SERVER_URI, bytes_mode=False)
		for opt, value in ldap_backend.settings.CONNECTION_OPTIONS.items():
			conn.set_option(opt, value)
		if ldap_backend.settings.START_TLS:
			conn.start_tls_s()
		conn.simple_bind_s(ldap_backend.settings.BIND_DN, ldap_backend.settings.BIND_PASSWORD)

		user_search = django.conf.settings.AUTH_LDAP_USER_SEARCH_ALL_NAME
		results = execute_ldap_search_without_hiding_errors(user_search, conn)
		ldap_usernames = [list(attr.values())[0][0] for dn, attr in results]
		django_users = get_user_model().objects.all()
		for username in ldap_usernames:
			print('creating or updating', username)
			# ldap_backend.populate_user also eats LDAP errors and returns None if
			# they occur. It also returns None if the user is not found in the LDAP
			# directory.
			user = ldap_backend.populate_user(username)
			if user is None:
				# If we end up here due to an LDAP error, it is most likely temporary.
				# Alternativly we end up here because the user was deleted in LDAP
				# after our initial user search (for ldap_usernames) (i.e. a race
				# condition, also temporary). So we abort and hope the next run of the
				# sync command works.
				raise Exception('LDAP lookup of a user failed for some reason, probably due to a connection error')
			user.save()
		for user in django_users:
			if not user.username in ldap_usernames:
				print('deactivating', user.username)
				user.is_active = False
				user.save()
